#!/usr/bin/env python

"""tasksscraper.trello: ...."""

__author__ = "Daniel Engvall"
__email__ = "daniel@engvalls.eu"

from trello import TrelloClient
from loguru import logger
from typing import List
# import keyring


def find_my_id(input_client, input_name):
    all_boards = input_client.list_boards()
    for b, board in enumerate(all_boards):
        all_cards = board.all_cards()
        for c, card in enumerate(all_cards):
            logger.debug(f'processing board {b}/{len(all_boards)}, card {c}/{len(all_cards)}')
            for member_id in card.member_ids:
                if input_name in input_client.get_member(member_id).full_name:
                    logger.info(f'found {input_name} as id {member_id}')
                    return member_id


def get_all_board_ids(input_client, input_tasks):
    all_ids = set()
    for t in input_tasks:
        all_ids.add(t['idBoard'])
    return {i: input_client.get_board(i).name for i in all_ids}


def add_project(input_my_tasks, board_ids):
    output_tasks = []
    for t in input_my_tasks:
        current_item = t
        current_item['project'] = board_ids[t['idBoard']]
        output_tasks.append(current_item)
    return output_tasks


def get_trello_tasks(api_key: str, token: str, token_secret: str, my_name: str) -> List:
    """Query Trello for tickets

    Args:
        my_name:
        api_key:
        token:
        token_secret:

    Returns:
        List of tasks

    """

    client = TrelloClient(
        api_key=api_key,
        api_secret=None,
        token=token,
        token_secret=token_secret
    )

    logger.info('fetching Trello tickets')

    my_id = find_my_id(client, my_name)
    my_cards = client.get_member(my_id).fetch_cards()
    board_ids = get_all_board_ids(client, my_cards)
    all_tickets = add_project(my_cards, board_ids)

    logger.info(f'complete fetching {len(all_tickets)} tickets')
    return all_tickets



from datetime import date as d
from pathlib import Path
from tasks_collector.tasksdb.api import get_default_db_path, insert_or_updates_tasks
from tasks_collector.tasksdb.api import OpenDB, cleanup
import peewee as pw
import pytest


def test_get_default_db_path(monkeypatch, tmp_path):
    monkeypatch.setattr('os.path.abspath', lambda x: tmp_path)
    monkeypatch.setattr('tasks_collector.tasksdb.api.user_config_dir', lambda x: tmp_path)
    assert Path(get_default_db_path()).parent.is_dir()


@pytest.fixture
def setup_db():
    db = pw.SqliteDatabase(':memory:')

    class Base(pw.Model):
        class Meta:
            database = db

    class Task(Base):
        subject = pw.CharField(unique=True)
        client = pw.CharField(null=True)
        category = pw.CharField(null=True)
        start_date = pw.DateField(null=True)
        close_date = pw.DateField(null=True)
        due_date = pw.DateField(null=True)
        modified_date = pw.DateField(null=True)
        status = pw.CharField(null=True)

    db.connect()
    db.create_tables([Task], safe=True)
    return db, Task


@pytest.fixture
def sample_task():
    t = {'subject': '[xxx-656] Helping xxx with escalations after xxx-Dev upgrade to 3.5',
         'client': 'xxx',
         'category': None,
         'start_date': '2018-11-14',
         'close_date': '2018-11-15',
         'due_date': '2018-11-15',
         'modified_date': '2018-11-15',
         'status': 'close'}
    return t


@pytest.fixture
def setup_initial_task(setup_db, mocker, sample_task):
    db, Task = setup_db
    db = OpenDB(':memory:')
    Task.get_or_create(subject='Foo Bar', defaults=sample_task)
    mocker.patch('tasks_collector.tasksdb.api.Task', Task)
    return db, Task


def test_insert_or_updates_tasks(setup_initial_task, sample_task):
    t2 = sample_task.copy()
    t2['subject'] = 'Foo Bar'
    task_list = [sample_task, t2]
    db, Task = setup_initial_task
    insert_or_updates_tasks(task_list)
    assert len(list(Task.select())) == 2


def test_get_all_tasks(setup_initial_task):
    db, Task = setup_initial_task
    all_tasks = db.get_all_tasks()
    assert 'xxx' in all_tasks[0]['subject']


def test_cleanup(setup_initial_task, sample_task):
    db, Task = setup_initial_task
    t2 = sample_task.copy()
    t2['subject'] = 'Foo Bar'
    t2['start_date'] = '2019-01-10'
    t2['close_date'] = None
    insert_or_updates_tasks([t2])
    cleanup('2019-01-20')
    assert Task.select().order_by(Task.id.desc()).get().close_date == d.today()



#!/usr/bin/env python

"""project_name: Small project for collecting tickets"""
from gooey import Gooey, GooeyParser
import argparse
import datetime as dt
import keyring
from getpass import getpass
import sys
from loguru import logger

from tasks_collector import reportgenerator
from tasks_collector.reportgenerator.api import filter_generic_tasks, tasks_to_pastebin, create_gantt_list
from tasks_collector.tasksconverter.api import to_generic
from tasks_collector.tasksdb.api import get_default_db_path, insert_or_updates_tasks, OpenDB, cleanup
from tasks_collector.tasksscraper.jirascraper import get_jira_tasks
from tasks_collector.tasksscraper.trelloscraper import get_trello_tasks
import tasks_collector.tasksscraper.outlookscraper

__author__ = "Daniel Engvall"
__email__ = "daniel@engvalls.eu"
__version__ = "0.9.5"


def get_keyring(system, username):
    password = keyring.get_password(system, username)
    if not password:
        password = getpass()
        keyring.set_password(system, username, password)
    return password


def get_args():
    default_db_path = get_default_db_path()
    app_description = 'A program for parsing any selected tasks\nitems in Outlook and/or Jira and generate report to pastebin'
    db_args = ('--sqlite_database',)
    db_kwargs = dict(help='name of sqlite to export/update to',
                     default=default_db_path)
    if not gui_disabled:
        db_kwargs['widget'] = 'FileChooser'
        argparser = GooeyParser(description=app_description)
    else:
        argparser = argparse.ArgumentParser(description=app_description)

    argparser.add_argument('--loglevel', default='INFO', choices=['INFO', 'DEBUG'])
    argparser.set_defaults(which='none')
    subparsers = argparser.add_subparsers(help='commands')
    collect_parser = subparsers.add_parser('collect')
    collect_parser.add_argument('--outlook', action='store_true')
    collect_parser.add_argument('--jira', help='username@jiraserver')
    collect_parser.add_argument('--trello', help='api_key:token:token_secret:my_name')
    # collect_parser.add_argument('--sqlite_database', help='name of sqlite to export/update to', default=default_db_path, widget='FileChooser')
    collect_parser.add_argument(*db_args, **db_kwargs)
    collect_parser.add_argument('--loglevel', default='INFO', choices=['INFO', 'DEBUG'])
    collect_parser.set_defaults(which='collect')
    report_parser = subparsers.add_parser('report')
    report_parser.add_argument('--days', type=int, default=10, metavar='number_of_days_in_past',
                           help='Number of days to cover in the report')
    report_parser.add_argument(*db_args, **db_kwargs)
    report_parser.add_argument('--copyq', action='store_true', help='paste output as MIME to pastebin, good for sending by e-mail')
    report_parser.add_argument('--show', action='store_true', help='show gantt image')
    report_parser.add_argument('--default_client', type=str, default='MyCompany', metavar='name of default client')
    report_parser.add_argument('--loglevel', default='INFO', choices=['INFO', 'DEBUG'])
    report_parser.set_defaults(which='report')
    cleanup_parser = subparsers.add_parser('cleanup')
    cleanup_parser.add_argument('--before', type=int, metavar='DAYS', help='tickets before this number of days back to be closed')
    cleanup_parser.add_argument('--sqlite_database', default=default_db_path, help='name of sqlite to export/update to')
    cleanup_parser.add_argument('--loglevel', default='INFO', choices=['INFO', 'DEBUG'])
    cleanup_parser.set_defaults(which='cleanup')
    args = argparser.parse_args()
    return args, default_db_path


def main():
    global gui_disabled
    gui_disabled = '--ignore-gooey' in sys.argv
    if gui_disabled:
        logger.info('no gui')
        sys.argv.remove('--ignore-gooey')
        args, default_db_path = get_args()
    else:
        logger.info('gui')
        # TODO: some tricky things need to be fixed to able to --ignore-gooey
        return Gooey(get_args, program_name='Tasks Collector', navigation='TABBED')()

    logger.remove()
    logger.add(sys.stderr, level=args.loglevel, colorize=not gui_disabled)

    if 'sqlite_database' not in args.__dict__.keys():
        db_path = default_db_path
    else:
        db_path = args.sqlite_database
    db = OpenDB(db_path)

    if 'days' in args.__dict__.keys():
        logger.info('report command initiated')
        now = dt.datetime.now()
        days = dt.timedelta(days=args.days)
        _from = (now - days).strftime('%Y-%m-%d')
        _to = (now + days).strftime('%Y-%m-%d')
        all_tasks = db.get_all_tasks()
        filtered_tasks = filter_generic_tasks(all_tasks, from_date=_from, to_date=_to)
        if args.copyq:
            tasks_to_pastebin(filtered_tasks, _filter=True, show_gantt=False, default_client=args.default_client)
        if args.show:
            gantt_list = create_gantt_list(filtered_tasks, args.default_client)
            reportgenerator.api.get_gantt_b64(gantt_list, show_gantt=True)
        sys.exit(0)

    if 'before' in args.__dict__.keys():
        logger.info('cleanup initiated')
        now = dt.datetime.now()
        days = dt.timedelta(days=args.__dict__['before'])
        _before = (now - days)
        logger.info(f'cleanup before {_before}')
        cleanup(_before)
        sys.exit(0)

    # Else fetch data
    if args.which == 'collect':
        logger.info('collection initiated!')
        generic_tasks = []
        if 'outlook' in args and args.outlook:
            outlook_tasks = tasks_collector.tasksscraper.outlookscraper.get_outlook_tasks()
            if not outlook_tasks:
                logger.warning('Unable to retrieve outlook tasks, make sure Outlook has been started')
            else:
                outlook_generic_tasks = to_generic(outlook_tasks, _type='outlook')
                generic_tasks.extend(outlook_generic_tasks)

        if 'jira' in args and args.jira:
            username, host = args.jira.split('@', 1)
            password = get_keyring('tasks_collector', username)
            jira_tasks = get_jira_tasks(host, username, password)
            jira_generic_tasks = to_generic(jira_tasks, _type='jira')
            generic_tasks.extend(jira_generic_tasks)

        if 'trello' in args and args.trello:
            api_key, token, token_secret, my_name = args.trello.split(':')
            trello_tasks = get_trello_tasks(api_key, token, token_secret, my_name)
            trello_generic_tasks = to_generic(trello_tasks, _type='trello')
            generic_tasks.extend(trello_generic_tasks)

        insert_or_updates_tasks(generic_tasks)


# noinspection PyPep8
if __name__ == '__main__':
    logger.remove()
    logger.add(sys.stderr, level="INFO", colorize=False)
    logger.info('Pass argument --ignore-gooey to use CLI')
    main()



#!/usr/bin/env python

"""tasksscraper.jira: ...."""

__author__ = "Daniel Engvall"
__email__ = "daniel@engvalls.eu"

import jira as j
from jira import JIRA
from loguru import logger
from typing import List
# import keyring


def get_jira_tasks(host: str, username: str, jira_password: str, max_results: int = 1000) -> List:
    """Query Jira for tickets

    Args:
        host:
        username:
        jira_password:
        max_results:

    Returns:
        List of tasks

    """
    # options = {'server': 'https://cog-jira.ipsoft.com', 'basic_auth': ('dengvall', pwd)}
    try:
        jira = JIRA(basic_auth=(username, jira_password), server=f'https://{host}')
    except j.exceptions.JIRAError:
        logger.error('Error connecting to server - please verify credentials')
        raise

    # Get all projects
    # projects = jira.projects()

    logger.info('fetching jira tickets')
    all_tickets = jira.search_issues('assignee = currentUser() order by priority desc', maxResults=max_results)
    logger.info(f'complete fetching {len(all_tickets)} tickets')
    return all_tickets



#!/usr/bin/env python

"""tasksconverter: ...."""

__author__ = "Daniel Engvall"
__email__ = "daniel@engvalls.eu"

import dateparser
from datetime import timedelta, datetime
from loguru import logger
from typing import List, Dict, Union, Optional


def format_subject(subject: str, _type='outlook') -> str:
    """Properly format subject

    Args:
        subject:
        _type:

    Returns:

    """
    import re
    # Highlight keywords
    subject = re.sub(r'(@\w+\([^\]]+\))', '<b>\\1</b>', subject)
    return subject


def parse_category(category_list: List, _type='outlook') -> Dict:
    """Parse categories

    Args:
        category_list:
        _type:

    Returns:

    """
    import re

    if _type == 'outlook':
        rules = {r'client': r'\(([^\}]+)\)',
                 r'category': r'\{([^\}]+)\}'}
        result = {}
        for _type in rules.keys():
            result.update({_type: None})
        for category in category_list:
            for _type in rules.keys():
                match = re.search(rules[_type], category)
                if match:
                    result.update({_type: match.group(1)})
        return result


def convert_date_attribute(date: Union[datetime, str]) -> Optional[str]:
    """Converting date attribute

    Args:
        date:

    Returns:

    """
    import datetime
    if type(date) is datetime.datetime:
        out_date = date.strftime('%Y-%m-%d')
    elif date is None:
        out_date = None
    else:
        parsed_date = dateparser.parse(str(date))
        if parsed_date:
            out_date = parsed_date.strftime('%Y-%m-%d')
        else:
            out_date = None
    return out_date


def correct_task(input_dict: Dict) -> Union[str, Dict]:
    """Correcting the task

    Args:
        input_dict:

    Returns:

    """
    i = input_dict
    logger.debug(f'correcting dates in {input_dict}')
    # make sure there is a start date
    start_date = dateparser.parse(str(i.get('start_date', None)))
    close_date = dateparser.parse(str(i.get('close_date', None)))
    if any(((not start_date and close_date), (start_date and close_date and close_date <= start_date))):
        i['start_date'] = (dateparser.parse(i['close_date']) - timedelta(days=1)).strftime('%Y-%m-%d')
    return i


# noinspection PyUnboundLocalVariable,PyUnboundLocalVariable
def to_generic(tasks_list: List, _type='outlook') -> List:
    """Make list of tasks generic

    Args:
        tasks_list:
        _type:

    Returns:
        New list of generic tasks

    """
    generic_list = list()
    for task in tasks_list:
        if _type == 'outlook':
            logger.debug('converting outlook to generic')
            subject = format_subject(task['taskName'], _type='outlook')
            categories = task['taskCategories']
            categories = parse_category(categories, _type='outlook')
            client, category = categories['client'], categories['category']
            due_date = convert_date_attribute(task['due'])
            start_date = convert_date_attribute(task['startDate'])
            modified_date = convert_date_attribute(task['modifiedDate'])
            close_date = convert_date_attribute(task['completeDate'])
            if close_date:
                status = 'close'
            else:
                status = 'open'
        elif _type == 'jira':
            logger.debug('converting jira to generic')
            key = task.key
            subject = f'[{key}] {task.fields.summary}'
            client = task.fields.project.name
            category = None
            start_date = convert_date_attribute(task.fields.created)
            close_date = convert_date_attribute(task.fields.resolutiondate)
            modified_date = convert_date_attribute(task.fields.updated)
            due_date = convert_date_attribute(task.fields.duedate)
            status = 'open' if not any([_ in task.fields.status.name for _ in ('Done', 'Not in Scope')]) else 'close'
            # fix None dates
            if not close_date and status == 'close':
                close_date = modified_date
            if not due_date:
                due_date = modified_date
        elif _type == 'trello':
            logger.debug('converting trello to generic')
            key = task['project']
            subject = f'[{key}] {task["name"]}'
            client = ''
            category = None
            status = 'open' if task['closed'] is False else 'close'
            due_date = convert_date_attribute(task['due'])
            start_date = dateparser.parse(task['dateLastActivity']) - timedelta(days=5)
            if due_date:
                start_date = (dateparser.parse(task['due']) - timedelta(days=5)).strftime('%Y-%m-%d')
            else:
                start_date = start_date.strftime('%Y-%m-%d')
            if status == 'open':
                close_date = None
            else:
                close_date = convert_date_attribute(task['dateLastActivity'])
            modified_date = convert_date_attribute(task['dateLastActivity'])


            # fix None dates
            if not close_date and status == 'close':
                close_date = modified_date
            if not due_date:
                due_date = modified_date

        generic_task = {'subject': subject,
                        'client': client,
                        'category': category,
                        'start_date': start_date,
                        'close_date': close_date,
                        'due_date': due_date,
                        'modified_date': modified_date,
                        'status': status}
        generic_task = correct_task(generic_task)
        generic_list.append(generic_task)
    # sort the list
    generic_list = sorted(generic_list, key=lambda x: (str(x['client']), str(x['category']), str(x['status'])))
    logger.debug('complete converting to generic')
    return generic_list




from tasks_collector.reportgenerator.api import dict_keys_to_ymd, render_task, \
    all_values, count_items, get_lowest_value, create_concurrent_list, create_gantt_list, filter_generic_tasks
import pytest


@pytest.fixture
def setup_invalid_date_task():
    t = {'subject': 'Foo Bar',
         'start_date': '2019-01-10',
         'close_date': '2019/01/10',
         'due_date': '1 january 2019',
         'modified_date': '2019-02-15 15:00'}
    return t


def test_dict_keys_to_ymd(setup_invalid_date_task):
    t = setup_invalid_date_task
    expected = {'close_date': '2019-01-10',
                'due_date': '2019-01-01',
                'modified_date': '2019-02-15',
                'start_date': '2019-01-10',
                'subject': 'Foo Bar'}
    result = dict_keys_to_ymd(t, ['start_date', 'close_date', 'due_date', 'modified_date'])
    assert result == expected


def test_render_task():
    t = render_task('my_client', 'my_category', 'my_subject', status='open')
    assert 'my_client' in t


@pytest.fixture
def setup_list_dict():
    return [{'name': 'Foo', 'value': 'x', 'number': 20}, {'name': 'Bar', 'value': 'y', 'number': 40}]


def test_all_values(setup_list_dict):
    assert all_values(setup_list_dict, key='name') == ['Bar', 'Foo']


def test_count_items(setup_list_dict):
    assert count_items(setup_list_dict, {'name': 'Foo'}) == 1


def test_get_lowest_value(setup_list_dict):
    assert get_lowest_value(setup_list_dict, 'number') == 40


@pytest.fixture
def setup_generic_task():
    return [{'subject': 'Foo Bar 1',
             'start_date': '2019-01-10',
             'close_date': None,
             'status': 'open',
             'due_date': '2019-01-12',
             'modified_date': '2019-01-10',
             'client_category': 'my_category_1',
             'client': 'client1',
             'category': 'my_category_1'},
            {'subject': 'Foo Bar 2',
             'start_date': '2019-02-10',
             'close_date': '2019-02-15',
             'due_date': None,
             'modified_date': '2019-02-15',
             'status': 'close',
             'client_category': 'my_category_2',
             'client': 'client2',
             'category': 'my_category_2'},
            {'subject': 'Foo Bar 3',
             'start_date': '2019-03-10',
             'close_date': None,
             'due_date': '2019-03-12',
             'modified_date': '2019-03-10',
             'status': 'open',
             'client_category': 'my_category_3',
             'client': 'client3',
             'category': 'my_category_3'}
            ]


def test_create_concurrent_list(setup_generic_task):
    expected = [{'date': '2019-01-10',
                 'my_category_1': 1,
                 'my_category_2': 0,
                 'my_category_3': 0},
                {'date': '2019-02-10',
                 'my_category_1': 1,
                 'my_category_2': 1,
                 'my_category_3': 0},
                {'date': '2019-02-15',
                 'my_category_1': 1,
                 'my_category_2': 0,
                 'my_category_3': 0},
                {'date': '2019-03-10',
                 'my_category_1': 1,
                 'my_category_2': 0,
                 'my_category_3': 1}]
    result = create_concurrent_list(setup_generic_task, name_key='client_category')
    assert result == expected


def test_create_gantt_list(setup_generic_task):
    expected = [
        ['client1:my_category_1:Foo Bar 1', '2019-01-10', '2019-01-12', 'over_due'],
        ['client2:my_category_2:Foo Bar 2', '2019-02-10', '2019-02-15', 'close'],
        ['client3:my_category_3:Foo Bar 3', '2019-03-10', '2019-03-12', 'over_due']]
    assert create_gantt_list(setup_generic_task, default_client='MyDefaultClient') == expected


@pytest.mark.parametrize('from_date,to_date,include_open,expected_len',
                         [('2019-01-01', '2019-01-20', False, 1),
                          ('2019-01-01', '2019-01-20', True, 2)])
def test_filter_generic_tasks(setup_generic_task, from_date, to_date, include_open, expected_len):
    result = filter_generic_tasks(setup_generic_task, from_date=from_date, to_date=to_date, include_open=include_open)
    assert len(result) == expected_len



import warnings

warnings.filterwarnings("ignore", category=DeprecationWarning)

import pytest
from tasks_collector.tasksconverter.api import to_generic, correct_task, format_subject, parse_category, convert_date_attribute
import json
from attrdict import AttrDict
import logging
from loguru import logger


@pytest.fixture
def caplog(caplog):
    class PropogateHandler(logging.Handler):
        def emit(self, record):
            logging.getLogger(record.name).handle(record)

    logger.add(PropogateHandler(), format="{message}")
    yield caplog


@pytest.mark.parametrize('input_file,input_type,output_dotted,expected_len',
                         [('tests/jira_tasks.json', 'jira', True, 2),
                          ('tests/outlook_tasks.json', 'outlook', False, 2),
                          ('tests/trello_tasks.json', 'trello', False, 2)])
def test_to_generic(input_file, input_type, output_dotted, expected_len, caplog):
    with open(input_file) as f:
        input_list = json.loads(f.read())
    if output_dotted:
        # used when the task being an object (dot notation) instead of a dict
        l = []
        for _ in input_list:
            l.append(AttrDict(_))
        input_list = l
    generic_list = to_generic(input_list, _type=input_type)
    assert len(generic_list) == expected_len, 'invalid number of records'
    assert 'complete converting to generic' in caplog.text, 'completion not found in log'


@pytest.mark.parametrize('input_dict,output_dict',
                         [({'start_date': '2018-01-20', 'close_date': '2018-01-10'},
                           {'start_date': '2018-01-09', 'close_date': '2018-01-10'}),
                          ({'close_date': '2018-01-15'},
                           {'start_date': '2018-01-14', 'close_date': '2018-01-15'}),
                          ({'start_date': '2018-02-23', 'close_date': '2018-02-27'},
                           {'start_date': '2018-02-23', 'close_date': '2018-02-27'}),
                          ])
def test_correct_task(input_dict, output_dict):
    assert correct_task(input_dict) == output_dict


def test_format_subject():
    assert format_subject('foo @bar(spam)', _type='outlook') == 'foo <b>@bar(spam)</b>'


def test_parse_category():
    assert parse_category(['(my_client) {my_category} some task']) == {'category': 'my_category',
                                                                       'client': 'my_client'}


@pytest.mark.parametrize('input_date,output', [('5/10/2019', '2019-05-10'),
                                               ('2019-01-01', '2019-01-01')])
def test_convert_date_attribute(input_date, output):
    assert convert_date_attribute(input_date) == output



#!/usr/bin/env python

"""tasksconverter: ...."""
import subprocess
from loguru import logger
import matplotlib
# workaround for MacOS
matplotlib.use('TkAgg')

import matplotlib.pyplot as plt
import matplotlib.font_manager as font_manager
import matplotlib.legend
# noinspection PyProtectedMember
from matplotlib.dates import WEEKLY, DateFormatter, rrulewrapper, RRuleLocator
import datetime as dt
import numpy as np
import base64
import json
import dateparser
import pandas as pd
from typing import Dict, List, Optional, Any

__author__ = "Daniel Engvall"
__email__ = "daniel@engvalls.eu"


html_header = u'''<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
<meta http-equiv="Content-Style-Type" content="text/css">
<title></title>
<meta name="Generator" content="Cocoa HTML Writer">
<meta name="CocoaVersion" content="1404.47">
<style type="text/css">
p.p1 {margin: 0.0px 0.0px 0.0px 0.0px; font: 14.0px Helvetica; -webkit-text-stroke: #000000}
p.p2 {margin: 0.0px 0.0px 0.0px 0.0px; font: 14.0px Helvetica; color: #017f00; -webkit-text-stroke: #017f00}
span.s1 {text-decoration: underline ; font-kerning: none}
span.s2 {font-kerning: none}
</style>
</head>
<body>
'''
html_footer = '''</body>
</html>'''


def dict_keys_to_ymd(_dict: Dict, _keys: List=[]) -> Dict:
    """Update keys with dates

    Args:
        _dict:
        _keys:

    Returns:
        New dict with parsed dates

    """
    for k in _keys:
        d = _dict[k]
        if d:
            parsed_date = dateparser.parse(str(d))
            _dict[k] = parsed_date.strftime('%Y-%m-%d')
    return _dict


def _create_date(datetxt: str) -> Any:
    """Create matplot lib date

    Args:
        datetxt:

    Returns:

    """
    year, month, day = datetxt.split('-')
    date = dt.datetime(int(year), int(month), int(day))
    mdate = matplotlib.dates.date2num(date)
    return mdate


# noinspection PyUnboundLocalVariable
def render_task(client: str, category: str, subject: str, **kwargs) -> str:
    """Render HTML for task

    Args:
        client:
        category:
        subject:

    Returns:

    """
    status = kwargs.get('status', 'open')
    close_date = kwargs.get('close_date', None)
    due_date = kwargs.get('due_date', None)
    start_date = kwargs.get('start_date', None)
    default_client = kwargs.get('default_client', 'None')
    suffix = '</span></p>'

    if status == u'open':
        prefix = u'<p class="p1"><span class="s2">     '
        char = u'☐'
    elif status == u'close':
        prefix = u'<p class="p2"><span class="s2">     '
        char = u'✔'

    # Add prefix and complete/not-complete character
    task = u'{}{}'.format(prefix, char)
    # Replace client with default if needed
    if not client:
        client = default_client
    task += u' <b>{}</b>: '.format(client)
    # Remove category if none
    if category:
        task += u'<i>{}:</i> '.format(category)
    # Add subject
    task += u'{}'.format(subject)
    # Add @start
    if start_date:
        task += u' <b>@start({})</b>'.format(start_date)
    # Add @due if needed
    if due_date:
        task += u' <b>@due({})</b>'.format(due_date)
    # Add @done
    if close_date:
        task += u' <b>@done({})</b>'.format(close_date)
    # Add suffix
    task += suffix
    return task


def create_concurrent_chart(concurrent_list, date_key='date', show_plot=False, concurrent_file='/tmp/concurrent.png', dpi=72):
    df = pd.DataFrame(concurrent_list)
    if len(df) == 0:
        raise RuntimeError('No dates in dataframe try more dates')
    df[date_key] = pd.to_datetime(df[date_key], format='%Y-%m-%d')
    df = df.sort_values(date_key, ascending=True)
    df.set_index([date_key], inplace=True)
    # Update resolution
    timedelta = pd.to_datetime(df.iloc[-1].name) - pd.to_datetime(df.iloc[0].name)
    if timedelta > pd.Timedelta(days=30):
        resolution = '1W'
    else:
        resolution = 'D'
    logger.debug(f'interpolating data to {resolution}')
    df = df.resample("D").mean()
    df = df.interpolate("nearest")
    df = df.resample(resolution).mean()
    logger.debug('plotting concurrent chart')

    # Get current size
    fig_size = plt.rcParams["figure.figsize"]
    fig_size[0] = 20
    fig_size[1] = 5
    plt.rcParams["figure.figsize"] = fig_size

    df.plot.area()
    plt.savefig(concurrent_file, dpi=dpi, bbox_inches='tight')
    if show_plot:
        plt.show()
    with open(concurrent_file, 'rb') as f:
        concurrent_b64 = base64.b64encode(f.read()).decode('utf-8')
    return concurrent_b64


def create_gantt_chart(task_list: List,
                       *,
                       show_plot: bool = True,
                       gantt_file: str = '/tmp/gantt.png',
                       dpi: int = 72) -> str:
    """Creates gantt chart

    Args:
        task_list:
        show_plot:
        gantt_file:
        dpi:

    Returns:
        A base64 encoded image

    """

    ylabels = []
    statuses = []
    custom_dates = []

    for t in task_list:
        ylabel, startdate, enddate, status = t
        statuses.append(status)
        ylabels.append(ylabel.replace('\n', ''))
        if enddate is None:
            enddate = dt.datetime.now() + dt.timedelta(days=7)
            enddate = enddate.strftime('%Y-%m-%d')
        custom_dates.append([_create_date(startdate.replace('\n', '')), _create_date(enddate.replace('\n', ''))])

    ilen = len(ylabels)
    pos = np.arange(0.5, ilen * 0.5 + 0.5, 0.5)
    task_dates = {}
    for i, task in enumerate(ylabels):
        task_dates[task] = custom_dates[i]
    fig = plt.figure(figsize=(20, 8))
    ax = fig.add_subplot(111)
    unique_legend_list = list()
    for i in range(len(ylabels)):
        start_date, end_date = task_dates[ylabels[i]]
        # Change color depending on status
        if statuses[i] == 'open':
            bar_color = 'orange'
        elif statuses[i] == 'pending':
            bar_color = 'gray'
        elif statuses[i] == 'over_due':
            bar_color = 'red'
        else:
            bar_color = 'green'
        label = statuses[i] if not statuses[i] in unique_legend_list else ""
        unique_legend_list.append(statuses[i])
        unique_legend_list = list(set(unique_legend_list))
        ax.barh((i * 0.5) + 0.5, end_date - start_date, left=start_date, height=0.3, align='center',
                edgecolor='lightgreen', color=bar_color, alpha=0.8, label=label)
    locsy, labelsy = plt.yticks(pos, ylabels)
    plt.setp(labelsy, fontsize=10)
    ax.set_ylim(ymin=-0.1, ymax=ilen * 0.5 + 0.5)
    ax.grid(color='g', linestyle=':')
    ax.xaxis_date()
    rule = rrulewrapper(WEEKLY, interval=1)
    loc = RRuleLocator(rule)
    formatter = DateFormatter("%d-%b")
    ax.xaxis.set_major_locator(loc)
    ax.xaxis.set_major_formatter(formatter)

    # Add legend
    ax.legend(loc='upper left')

    # Add red line current date
    conv = np.vectorize(matplotlib.dates.strpdate2num('%Y-%m-%d'))
    now = dt.datetime.now().strftime('%Y-%m-%d')
    # ax.axvline(conv(now), color='r', zorder=0)
    ax.axvline(float(conv(now)), color='r', zorder=0)

    labelsx = ax.get_xticklabels()
    plt.setp(labelsx, rotation=30, fontsize=10)

    font = font_manager.FontProperties(size='small')
    ax.legend(loc=1, prop=font)

    ax.invert_yaxis()
    fig.autofmt_xdate()
    plt.savefig(gantt_file, dpi=dpi, bbox_inches='tight')
    if show_plot:
        plt.show()
    with open(gantt_file, 'rb') as f:
        gantt_b64 = base64.b64encode(f.read()).decode('utf-8')
    return gantt_b64


def tasks_to_pastebin(generic_tasks: List, _filter: bool = False, show_gantt: bool = True, default_client='None') -> None:
    """Creates tasks and inserted to pastebin

    Args:
        default_client:
        generic_tasks:
        _filter:
        show_gantt:

    Returns:

    """
    # from_date = kwargs.get('from_date', None)
    # to_date = kwargs.get('to_date', None)
    # include_open = kwargs.get('include_open', True)

    logger.debug(json.dumps(generic_tasks))
    email_html = html_header

    for t in generic_tasks:
        # Render HTML for tasks
        email_html += render_task(t['client'],
                                  t['category'],
                                  t['subject'],
                                  status=t['status'],
                                  close_date=t['close_date'],
                                  due_date=t['due_date'],
                                  start_date=t['start_date'],
                                  default_client=default_client)

    # Render gantt
    gantt_list = create_gantt_list(generic_tasks, default_client)
    gantt_b64 = get_gantt_b64(gantt_list, show_gantt)
    # Attach image
    email_html += '<img tasks_collector="data:image/png;base64,{}" alt="gantt.png">'.format(gantt_b64)


    # Render concurrance chart
    for t in generic_tasks:
        t.update({'client_category': '{}_{}'.format(t['client'], t['category'])})
    concurrent_list = create_concurrent_list(generic_tasks, name_key='client_category')
    concurrent_b64 = create_concurrent_chart(concurrent_list, date_key='date', show_plot=show_gantt)
    email_html += '<img tasks_collector="data:image/png;base64,{}" alt="concurrent.png">'.format(concurrent_b64)

    # Add footer
    email_html += html_footer

    logger.debug(email_html)

    proc = subprocess.Popen(['/usr/local/bin/copyq', 'copy', 'text/html', '-'],
                            stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = proc.communicate(input=email_html.encode('utf-8'))
    logger.debug(out, err)


def get_gantt_b64(gantt_list: List, show_gantt: bool = True) -> str:
    """Returns base64 encoded string of gantt

    Args:
        gantt_list:
        show_gantt:

    Returns:

    """
    gantt_b64 = create_gantt_chart(gantt_list, show_plot=show_gantt)
    return gantt_b64


def all_values(in_data: List, key: str = 'name') -> List:
    """Create sorted list

    Args:
        in_data:
        key:

    Returns:

    """
    return sorted(list(set([str(_[key]) for _ in in_data])))


def count_items(in_data: List, dict_condition: Dict) -> int:
    """Count items

    Args:
        in_data:
        dict_condition:

    Returns:

    """
    count = 0
    for item in in_data:
        condition_met = all([item.get(k, None) == v for k, v in dict_condition.items()])
        if condition_met:
            count += 1
    return count


def get_lowest_value(input_dict_list: Dict, key_name: str) -> int:
    """Get lowest value

    Args:
        input_dict_list:
        key_name:

    Returns:

    """
    lowest = 0
    for d in input_dict_list:
        current_value = d[key_name]
        if type(current_value) is int:
            if current_value >= lowest:
                lowest = current_value
    return lowest


def create_concurrent_list(in_data: List,
                           name_key: str = 'name',
                           start_key: str = 'start_date',
                           end_key: str = 'close_date') -> List:
    """Creates list of concurrent tasks to generate plot of

    Args:
        in_data:
        name_key:
        start_key:
        end_key:

    Returns:
        List to generate plots

    """
    plot_data = []
    all_dates = all_values(in_data, key=start_key) + all_values(in_data, key=end_key)
    all_dates = [_ for _ in all_dates if not _ == 'None']
    all_names = all_values(in_data, key=name_key)
    counter = {name: 0 for name in all_names}
    for date in sorted(all_dates):
        count_dict = {'date': date}
        for name in all_names:
            count_start = count_items(in_data, {start_key: date, name_key: name})
            count_end = count_items(in_data, {end_key: date, name_key: name})
            count = count_start - count_end
            # Prevent going below 0 when there is discrepancy
            counter[name] += count
            if counter[name] < 0:
                counter[name] = 0
            count_dict[name] = counter[name]
        plot_data.append(count_dict)
    return plot_data


def create_gantt_list(generic_tasks: List, default_client='None') -> List:
    """Creates a list formatting tasks names etc

    Args:
        default_client:
        generic_tasks:

    Returns:
        New list

    """
    gantt_list = []
    for t in generic_tasks:
        # Create data for gantt
        client = default_client if not t['client'] else t['client']
        gantt_task_name = '{}:{}:{}'.format(client, t['category'], t['subject'][:30]) if t[
            'category'] else '{}:{}...'.format(client, t['subject'][:30])
        if not gantt_task_name:
            gantt_task_name = default_client
        gantt_status = t['status'] if not all(
            [not t['start_date'], not t['close_date'], not t['due_date']]) else 'pending'
        gantt_start_date = t['start_date'] if t['start_date'] else t['modified_date']
        gantt_end_date = t['close_date'] if t['close_date'] else t['due_date']
        if gantt_end_date:
            if gantt_end_date == gantt_start_date:
                gantt_end_date = (dt.datetime.strptime(gantt_end_date, '%Y-%m-%d') + dt.timedelta(days=1)).strftime(
                    '%Y-%m-%d')
            if not t['due_date'] is None:
                if dt.datetime.strptime(t['due_date'], '%Y-%m-%d') < dt.datetime.now() and not gantt_status == 'close':
                    gantt_status = 'over_due'
        gantt_list.append([gantt_task_name, gantt_start_date, gantt_end_date, gantt_status])
    return gantt_list


# noinspection PyPep8
def filter_generic_tasks(generic_task_list: List,
                         *,
                         from_date: Optional[str] = None,
                         to_date: Optional[str] = None,
                         include_open: bool = True) -> List:
    """Filter generic tasks based on criterias

    Args:
        generic_task_list:
        from_date:
        to_date:
        include_open:

    Returns:
        A new task list

    """
    import datetime
    result = []
    for t in generic_task_list:
        dict_keys_to_ymd(t, ['start_date', 'close_date', 'due_date', 'modified_date'])
        include_ticket = False
        if include_open and t['status'] == 'open':
            include_ticket = True
        elif all([from_date, to_date]):
            # noinspection PyPep8
            if datetime.datetime.strptime(from_date, '%Y-%m-%d') <= datetime.datetime.strptime(t['start_date'],
                                                                                               '%Y-%m-%d') <= datetime.datetime.strptime(
                    to_date, '%Y-%m-%d'):
                include_ticket = True
        if include_ticket:
            result.append(t)
    return result



#!/usr/bin/env python

"""tasksscraper.outlook: ...."""

__author__ = "Daniel Engvall"
__email__ = "daniel@engvalls.eu"

import osascript
from typing import List
import json
from loguru import logger
import re


def remove_invalid_brackets(dirty_json):
    """Sanitize JSON from extra brackets caused by AppleScript

    Args:
        dirty_json:

    Returns: String without extra brackets

    """
    output_data = dirty_json.replace('\n', '')
    re_rules = [(r'(?s),[\{]{2,}', r',{')]
    current_count = 1
    max_count = 100  # TODO: a nasty workaround for weird behavior found using Python 3.8
    all_ok = False
    while not all_ok and current_count <= max_count:
        for replace_match, replace_with in re_rules:
            output_data = re.sub(replace_match, replace_with, output_data, re.DOTALL)
        all_ok = all([re.match(r, output_data) for r, _ in re_rules])
        current_count += 1
    output_data = output_data.replace('{{{', '{')
    return output_data


def fix_quotes_json_strings(dirty_json):
    """Replace double-quotes with JSON strings causing issues

    Args:
        dirty_json:

    Returns: Returns a string without invalid quotes

    """
    output_data = dirty_json
    all_tasks = re.findall(r'"taskName":"(.+?)","', dirty_json)
    fix_tasks = [(t, t.replace('"', "'")) for t in all_tasks if '"' in t]
    print(fix_tasks)
    for fix_from, fix_to in fix_tasks:
        print(f'{fix_from} {fix_to}')
        output_data = output_data.replace(fix_from, fix_to)
    return output_data


def get_outlook_tasks() -> List:
    """Get Outlook tasks

    Returns:
        List of tasks from Outlook

    """
    # noinspection PyPep8,PyPep8
    c, o, e = osascript.run('''
    -- display dialog "Will now process all current outlook tasks selected"
    tell application "Microsoft Outlook"
        -- activate
        set x to ""
        set selectedTasks to selected objects
        if selectedTasks is {} then
            display dialog "Please select a task first and then run this script." with icon 1
            return
        end if
        repeat with theTask in selectedTasks
            set c to "{"
            if class of theTask is task then
                set c to c & "\\"taskName\\":\\"" & name of theTask & "\\""
                set theContent to plain text content of theTask
                if theContent is missing value then
                    set theContent to ""
                end if
                set c to c & ",\\"taskContent\\":\\"" & "none" & "\\""
                set theFolder to folder of theTask
                set c to c & ",\\"taskFolder\\":\\"" & name of theFolder & "\\""
                set c to c & ",\\"modifiedDate\\":\\"" & modification date of theTask & "\\""
                set c to c & ",\\"startDate\\":\\"" & start date of theTask & "\\""
                set c to c & ",\\"due\\":\\"" & due date of theTask & "\\""
                set c to c & ",\\"completeDate\\":\\"" & completed date of theTask & "\\""
                set c to c & ",\\"taskPriority\\":\\"" & priority of theTask & "\\""
                set theCategory to category of theTask
                set catList to "["
                set cat_count to 0 as number
                repeat with oneCat in theCategory
                    if oneCat is not equal to ""
                        set theCategoryName to name of oneCat
                        if cat_count > 0 then
                            set catList to (catList & ",") as string
                        end if
                        set catList to catList & "\\"" & theCategoryName & "\\""
                        set cat_count to cat_count + 1 as number
                    end if
                end repeat
                set catList to catList & "]"
                set c to c & ",\\"taskCategories\\":" & catList
                set c to c & "},"
            end if
        set x to (x & c) as string
        end repeat
    end tell
    return x
    ''') # nopep8
    o = o.strip(',')
    o = f'[{o}]'
    logger.info(f'osascript error: {e}')
    logger.info(f'osascript output: {o}')
    logger.info(f'sanitizing JSON generated by AppleScript')
    logger.debug(f'result after sanitizing {o}')
    o = remove_invalid_brackets(o)
    o = fix_quotes_json_strings(o)
    if len(o) <= 10:
        logger.warning('less than 10 characters from Outlook, probably nothing there. Skipping Outlook')
    if c == 0:
        try:
            result = json.loads(o)
        except json.decoder.JSONDecodeError:
            logger.opt(exception=True).error('unable to parse JSON from Outlook - skipping Outlook, see the body printed')
            print(o)
            return None
        return result
    else:
        return None


from tasks_collector.tasksscraper.outlookscraper import get_outlook_tasks, remove_invalid_brackets, fix_quotes_json_strings
import json


def test_get_outlook_tasks(mocker):
    with open('tests/outlook_tasks.json') as f:
        input_list = json.loads(f.read())
    input_list = (0, '{"taskName": "Third task",'
                     '"taskContent": "","taskFolder": '
                     '"Tasks","modifiedDate": '
                     '"Sunday, 24 November 2019 at 20:09:04",'
                     '"startDate": "Saturday, 23 November 2019 at 00:00:00",'
                     '"due": "Friday, 29 November 2019 at 00:00:00",'
                     '"completeDate": "missing value",'
                     '"taskPriority": "priority normal",'
                     '"taskCategories": ["{Project X}","(Client 1)"]},'
                     '{"taskName": "The second task",'
                     '"taskContent": "",'
                     '"taskFolder": "Tasks",'
                     '"modifiedDate": "Sunday, 24 November 2019 at 22:39:00",'
                     '"startDate": "Monday, 16 September 2019 at 00:00:00",'
                     '"due": "Thursday, 19 September 2019 at 00:00:00",'
                     '"completeDate": "Sunday, 24 November 2019 at 22:39:00",'
                     '"taskPriority": "priority normal",'
                     '"taskCategories": ["{Project Y}","(Client 1)"]},'
                     '{"taskName": "My first task",'
                     '"taskContent": "","taskFolder": '
                     '"Tasks","modifiedDate": "Sunday, 24 November 2019 at 20:09:36",'
                     '"startDate": "Tuesday, 3 September 2019 at 00:00:00",'
                     '"due": "Wednesday, 11 September 2019 at 00:00:00",'
                     '"completeDate": "missing value",'
                     '"taskPriority": "priority normal",'
                     '"taskCategories": ["(Client 2)"]},', '')
    mocked_outlook = mocker.patch('tasks_collector.tasksscraper.outlookscraper.osascript')
    mocked_outlook.run.return_value = input_list
    all_tasks = get_outlook_tasks()
    assert mocked_outlook.run.call_count == 1
    assert len(all_tasks) == 3


def test_remove_invalid_brackets():
    invalid_json = '[{"foo":"bar"},{{{"bar":"foo"}]'
    assert remove_invalid_brackets(invalid_json) == '[{"foo":"bar"},{"bar":"foo"}]'


def test_fix_quotes_json_strings():
    invalid_json = '[{"taskName":"invalid "quotes" here","taskPriority":"priority normal"}]'
    assert fix_quotes_json_strings(invalid_json) == """[{"taskName":"invalid 'quotes' here","taskPriority":"priority normal"}]"""



from tasks_collector.tasksscraper.jirascraper import get_jira_tasks
import json

def test_get_jira_tasks(mocker):
    with open('tests/jira_tasks.json') as f:
        input_list = json.loads(f.read())
    mocked_jira = mocker.patch('tasks_collector.tasksscraper.jirascraper.JIRA')
    mocked_jira.return_value.search_issues.return_value = input_list
    all_tasks = get_jira_tasks('server', 'username', 'password')
    assert mocked_jira.call_count == 1
    assert len(all_tasks) == 2



from setuptools import setup, find_packages
import os

opt = {
    'install_requires': {'in': (True, ['requirements.txt', '../requirements.txt']), 'out': None},
    'long_description': {'in': (False, ['README.md', '../README.md']), 'out': None}
}

for k, v in opt.items():
    _split, files = v['in']
    for p in files:
        try:
            with open(p) as f:
                d = f.read()
                opt[k]['out'] = d.splitlines() if _split else d
                print(f'Found data for {k}')
            break
        except FileNotFoundError:
            print(f'Unable to find {p} being in {os.getcwd()}')
    if not opt[k]['out']:
        raise RuntimeError(f'Abort due to missing data for {k} in setup')

setup(
    name='tasks_collector',
    version='0.9.6',
    packages=find_packages(),
    install_requires=opt['install_requires']['out'],
    url='https://github.com/engdan77/tasks_collector',
    license='MIT',
    author='Daniel Engvall',
    author_email='daniel@engvalls.eu',
    description='A small project for collecting tasks',
    long_description=opt['long_description']['out'],
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Environment :: MacOS X",
        "Topic :: Office/Business"
    ],
    entry_points={
        'console_scripts': ['tasks_collector = tasks_collector.__main__:main'],
        'gui_scripts': ['tasks_collector_gui = tasks_collector.__main__:main']
        }
)



#!/usr/bin/env python

"""tasksdb: ...."""

import peewee as pw
from loguru import logger
import datetime
from dictdiffer import diff
from appdirs import user_config_dir
from pathlib import Path
import os
from typing import Dict, List

db = pw.SqliteDatabase(None)
now = datetime.datetime.now()


# Database model
class BaseModel(pw.Model):
    class Meta:
        database = db


class Task(BaseModel):
    subject = pw.CharField(unique=True)
    client = pw.CharField(null=True)
    category = pw.CharField(null=True)
    start_date = pw.DateField(null=True)
    close_date = pw.DateField(null=True)
    due_date = pw.DateField(null=True)
    modified_date = pw.DateField(null=True)
    status = pw.CharField(null=True)


def get_default_db_path() -> str:
    """Get a default database path

    Returns:
        str: path
    """
    p = globals().get('__package__')
    local_config_dir = next(iter([d / 'config' for d in list(Path(os.path.abspath(__file__)).parents)[:2] if (d / 'config').exists()]), None)
    if local_config_dir and local_config_dir.exists():
        logger.debug(f'found local config directory in {local_config_dir}')
        base_config_dir = local_config_dir
    else:
        base_config_dir = user_config_dir(p)
        logger.debug(f'no local config directory, directory will be used {base_config_dir}')
    conf_path = Path(base_config_dir) / Path(f'{p}.sqlite')
    conf_path.parent.mkdir(parents=True, exist_ok=True)
    return str(conf_path)


def get_kv_task_as_text(task: Task, remove_keys: List=['id']) -> Dict:
    t = dict([(k, v.strftime('%Y-%m-%d')) if isinstance(v, datetime.date) else (k, v) for k, v in task.items()])
    for _id in remove_keys:
        t.pop(_id)
    return t


def insert_or_updates_tasks(tasks_list_dict: List) -> None:
    """Insert or update task

    Args:
        tasks_list_dict: List of tasks to update

    Returns:
        None

    """
    for t in tasks_list_dict:
        # only works with sqlite 3.24.0
        # rowid = (Task
        #          .insert(subject=t['subject'], status=t['status'])
        #          .on_conflict(conflict_target=[Task.subject],
        #     update={Task.status: t['status']},
        #     preserve=[Task.subject]).execute())

        try:
            task, created = Task.get_or_create(subject=t['subject'], defaults=t)
            database_record = task.__dict__['__data__']
        except pw.IntegrityError as e:
            logger.warning(f'unable to create database row for {t} due to {e}')
            created = False
        else:
            if not created:
                dict_database_record = get_kv_task_as_text(database_record)
                dict_new_record = t
                if not dict_database_record == dict_new_record:
                    logger.info(f'updating record "{t["subject"]}" {list(diff(dict_database_record, dict_new_record))}')
                    Task.update(t).where(Task.subject == t['subject']).execute()
        logger.debug(f'record created: {created}')


def cleanup(before_date):
    """Cleanup the database

    Args:
        before_date:

    Returns:
        None

    """
    tasks = Task.select().where((Task.start_date <= before_date) & (Task.status == 'open'))
    logger.info('closing following tasks:')
    for t in tasks:
        logger.info(t.subject)
    tasks = Task.update(status='close').where((Task.start_date <= before_date) & (Task.status == 'open')).execute()

    logger.info('closing following closed tasks without close date closing today:')
    tasks = Task.select().where((Task.start_date <= before_date) & (Task.status == 'close') & (Task.close_date == None))
    for t in tasks:
        logger.info(t.subject)
    Task.update(close_date=datetime.datetime.now()).where((Task.start_date <= before_date) & (Task.status == 'close') & (Task.close_date == None)).execute()
    logger.info('cleanup complete')


class OpenDB(object):
    """A class to simplify creation of a database."""
    def __init__(self, db_file, _type='sqlite'):
        db.init(db_file)
        db.connect()
        db.create_tables([Task], safe=True)
        self.db = db

    @staticmethod
    def get_all_tasks() -> List:
        all_tasks = []
        for task in Task.select():
            t = task.__dict__['__data__']
            t.pop('id')
            all_tasks.append(t)
        return all_tasks



from tasks_collector.tasksscraper.trelloscraper import add_project, get_trello_tasks
import json


def test_add_project():
    input_my_tasks = [{'project': 'myProject', 'idBoard': 1}]
    board_ids = {1: 'boardName'}
    assert add_project(input_my_tasks, board_ids) == [{'idBoard': 1, 'project': 'boardName'}]


def test_get_trello_tasks(mocker):
    with open('tests/trello_tasks.json') as f:
        input_list = json.loads(f.read())
    mocker.patch('tasks_collector.tasksscraper.trelloscraper.TrelloClient')
    mocker.patch('tasks_collector.tasksscraper.trelloscraper.find_my_id')
    mocker.patch('tasks_collector.tasksscraper.trelloscraper.get_all_board_ids')
    mocked_add_project = mocker.patch('tasks_collector.tasksscraper.trelloscraper.add_project')
    mocked_add_project.return_value = input_list
    trello_tasks = get_trello_tasks('api_key', 'token', 'token_secret', 'my name')
    assert len(trello_tasks) == 2
    assert 'RFC' in trello_tasks[0]['desc']